
import os
import subprocess
import sys
import test_config
import traceback
import time

class Test:
	RESULT_NONE = 0
	RESULT_SUCCESS = 1
	RESULT_FAILED = 2

	def __init__(self):
		self.result = RESULT_NONE
		self.name = "undefined_test"
		self.stdout = ""
		self.stderr = ""

	def run(self):
		print "test", self.name, "not implemented"
		self.result = RESULT_FAILED

	def cmp(self):
		print "test", self.name, "has no -cmp method implemented"

	def new(self):
		print "test", self.name, "has no -new method implemented"

	def old(self):
		print "test", self.name, "has no -old method implemented"
	
	def _run_blender(self, path, args):
		proc = subprocess.Popen(["../../" + path] + args,
			stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		out, err = proc.communicate()

		self.stdout += out + err
		self.stderr += err

		return proc.returncode
	
	def run_blender(self, args):
		return self._run_blender(test_config.BLENDER, args)

	def run_old_blender(self, args):
		return self._run_blender(test_config.OLD_BLENDER, args)
	
	def fail(self, reason):
		self.stdout += "\nTEST FAILED: " + reason + "\n"
		self.result = self.RESULT_FAILED
	
	def success(self):
		self.result = self.RESULT_SUCCESS

	def _enter_dir(self):
		tmpdir = "tmp/" + self.name + ".dir"

		if not os.path.exists(tmpdir):
			os.mkdir(tmpdir)

		os.chdir(tmpdir)

	def _exit_dir(self):
		os.chdir("../..")

	# wrappers

	def cmd_run(self, i, tot):
		print "-- [%d/%d] " % (i, tot) + self.name + " ",
		sys.stdout.flush()

		cwd = os.getcwd()
		dt = time.time()
		self._enter_dir()
		try:
			self.run()
		except:
			self.stdout += traceback.format_exc()
			self.result = self.RESULT_FAILED
		self._exit_dir()
		dt = time.time() - dt
		os.chdir(cwd)

		print "\r-- [%d/%d] " % (i, tot) + self.name,
		# print "[%.3lf" % dt + "s]",

		OK = "\033[92m"
		WARNING = "\033[93m"
		FAIL = "\033[91m"
		ENDC = "\033[0m"

		if self.result == self.RESULT_FAILED:
			print "[" + FAIL + "failed" + ENDC + "]",
		elif self.result == self.RESULT_SUCCESS:
			print "[" + OK + "success" + ENDC + "]",
		elif self.result:
			print "[" + FAIL + "error: no result specified" + ENDC + "]",
		
		if len(self.stderr) > 0:
			lines = len(self.stderr.splitlines())
			print "[" + WARNING + str(lines) + " stderr lines" + ENDC + "]",
		
		print ""

		tmpdir = "tmp/" + self.name + ".dir"
		if not os.path.exists(tmpdir):
			os.mkdir(tmpdir)

		f = open(tmpdir + "/log", "w")
		f.write(self.stdout)
		f.close()

	def cmd_cmp(self):
		self._enter_dir()
		self.cmp()
		self._exit_dir()

	def cmd_log(self):
		f = open("tmp/" + self.name + ".dir/log", "r")
		lines = f.readlines()
		for line in lines:
			print line,
		f.close()

	def cmd_new(self):
		self._enter_dir()
		self.new()
		self._exit_dir()

	def cmd_old(self):
		self._enter_dir()
		self.old()
		self._exit_dir()

class TestCollection:
	def __init__(self):
		self.name = "undefined_collection"
		self.tests = []

