
import os
import shutil
import subprocess
import struct
import test_config
import test_lib

class Image:
	def __init__(self, filename):
		# simple, stupid TGA file reading, code purely
		# for the images written by render_compare.blend
		f = open(filename, "rb")
		header = struct.unpack('bbb', f.read(3))
		header1 = struct.unpack('hhb', f.read(5))
		header2 = struct.unpack("hhhhbb", f.read(10))

		f.read(header[0])
		f.read(header[1])

		self.w = header2[2]
		self.h = header2[3]

		self.pixels = f.read(self.w*self.h*header2[4])

		f.close()
	
	def test_empty(self, test):
		max = 0

		for v in self.pixels:
			if abs(ord(v)) > max:
				max= abs(ord(v))

		if max > test_config.RENDER_PIXEL_LIMIT*255:
			test.stdout += "\nPIXEL DIFFERENCE FOUND: " + str(max) + "\n"
			return 0

		return 1

class RenderTest(test_lib.Test):
	def __init__(self, dir, name):
		self.name = name
		self.filename= os.getcwd() + "/" + dir + "/" + name
		self.stdout= ""
		self.stderr= ""
	
	def run(self):
		filename = self.filename

		retval1 = self.run_blender(["-b", filename, "-o", "new", "-F", "EXR", "-x", "1", "-f", "1"])
		retval2 = self.run_old_blender(["-b", filename, "-o", "old", "-F", "EXR", "-x", "1", "-f", "1"])

		if retval1 != 0:
			return self.fail("process returned value " + str(retval1))
		if retval2 != 0:
			return self.fail("process returned value " + str(retval2))

		# compare results
		self.run_blender(["-b", "../../util/render_compare.blend", "-o", "diff", "-F", "RAWTGA", "-x", "1", "-f", "1"])

		img = Image("diff0001.tga")
		if not img.test_empty(self):
			return self.fail("pixel values are different")

		# and we're done
		return self.success()
	
	def cmp(self):
		self.run_old_blender(["util/render_compare.blend"])

	def new(self):
		self.run_blender([self.filename])

	def old(self):
		self.run_old_blender([self.filename])
	
class RenderTestCollection(test_lib.TestCollection):
	def __init__(self):
		test_lib.TestCollection.__init__(self)
		self.name = "render"

		for root, dirs, files in os.walk(test_config.RENDER_DIR):
			for file in files:
				if file.endswith(".blend"):
					self.tests += [RenderTest(root, file)]

